@@ -133,6 +133,26 @@ def add_arguments(parser):
         help="if set to yes, warn various kinds of exceptions and mistakes and log them to a log file with line number in input file, rather than stopping. logging",
         dest="warning",
     )
+    parser.add_argument(
+        "-i",
+        "--input-file",
+        action="store",
+        type=str,
+        required = False,
+        default="",
+        help="set the path of the input kgtk file if not from standard input",
+        dest="input_file",
+    )
+    parser.add_argument(
+        "-r",
+        "--rank",
+        action="store",
+        type=bool,
+        required = False,
+        default=False,
+        help="Whether the input file contains a rank column. Please refer to the `import_wikidata` command for the header information. Default to False, then all the ranks will be `normal`, therefore `NormalRank`.",
+        dest="has_rank",
+    )
 
 
 def run(
@@ -145,12 +165,15 @@ def run(
     output_prefix: str,
     n: int,
     log_path: str,
-    warning: bool
+    warning: bool,
+    input_file: str,
+    has_rank:bool,
 ):
     # import modules locally
     from kgtk.generator import JsonGenerator
     import sys
     import gzip
+    from kgtk.exceptions import KGTKException
     
     generator = JsonGenerator(
         label_set=labels,
@@ -162,39 +185,59 @@ def run(
         log_path = log_path,
         warning = warning,
         prop_declaration = prop_declaration,
+        has_rank = has_rank
     )
-    # process stdin
+    # loop first round
     if use_gz:
-        fp = gzip.open(sys.stdin.buffer, 'rt')
+        if input_file:
+            try:
+                fp = open(input_file,"rb")
+            except:
+                raise KGTKException("Fail to read from compressed file {}. Exiting.".format(input_file))
+        else:
+            fp = gzip.open(sys.stdin.buffer, 'rt')
     else:
-        fp = sys.stdin
+        if input_file:
+            try:
+                fp = open(input_file,"r")
+            except:
+                raise KGTKException("Fail to read from file {}. Exiting.".format(input_file))
+        else:
+            fp = sys.stdin
         # not line by line
 
     if prop_declaration:
-        file_lines = 0
-        begining_edge = None
-        start_generation = False
-        for line_num, edge in enumerate(fp):
-            if line_num == 0:
-                begining_edge = edge
+        if input_file:
+            for line_num, edge in enumerate(fp):
+                generator.read_prop_declaration(line_num+1,edge)
+            fp.seek(0)
+            for line_num, edge in enumerate(fp):
                 generator.entry_point(line_num+1,edge)
-                file_lines += 1
-                # print("initial edge at line {}".format(line_num))
-            else:
-                if start_generation:
-                    # start triple generation because reached the starting position of the second `cat`
-                    line_number = line_num - file_lines
-                    # print("creating jsons at line {} {} with total number of lines: {}".format(line_number+1, edge, file_lines))
-                    generator.entry_point(line_number+1,edge) # file generator
-                    # print("# {}".format(generator.read_num_of_lines))
+        else:
+            file_lines = 0
+            begining_edge = None
+            start_generation = False
+            for line_num, edge in enumerate(fp):
+                if line_num == 0:
+                    begining_edge = edge
+                    generator.entry_point(line_num+1,edge)
+                    file_lines += 1
+                    # print("initial edge at line {}".format(line_num))
                 else:
-                    if edge == begining_edge:
-                        # print("set generation start at line {} {}".format(line_num, edge))
-                        start_generation = True
+                    if start_generation:
+                        # start triple generation because reached the starting position of the second `cat`
+                        line_number = line_num - file_lines
+                        # print("creating jsons at line {} {} with total number of lines: {}".format(line_number+1, edge, file_lines))
+                        generator.entry_point(line_number+1,edge) # file generator
+                        # print("# {}".format(generator.read_num_of_lines))
                     else:
-                        file_lines += 1
-                        # print("creating property declarations at line {} {}".format(line_num, edge))
-                        generator.read_prop_declaration(line_num+1,edge)
+                        if edge == begining_edge:
+                            # print("set generation start at line {} {}".format(line_num, edge))
+                            start_generation = True
+                        else:
+                            file_lines += 1
+                            # print("creating property declarations at line {} {}".format(line_num, edge))
+                            generator.read_prop_declaration(line_num+1,edge)
         generator.finalize()
     else:
         for line_num, edge in enumerate(fp):
@@ -204,3 +247,5 @@ def run(
                 generator.entry_point(line_num+1,edge)
     
         generator.finalize()
+    if input_file:
+        fp.close()
