@@ -22,9 +22,26 @@ The output file contains the union of the columns in the two
 input files, adjusted for predefined name aliasing.
 
 Specify --left-join to get a left outer join.
+        The output file will contain all records from the
+        left input file, along with records from the right
+        input file with matching join column values.
+
 Specify --right-join to get a right outer join.
-Specify both to get a full outer join (equivalent to cat).
-Specify neither to get an inner join.
+        The output file will contain all records from the
+        right input file, along with records from the left
+        input file with matching join column values.
+
+Specify both --left-join and --right-join to get a full outer
+join (equivalent to cat or set union).
+        The output file will contain all records from both
+        the left input file and the right input file.
+
+Specify neither --left-join nor --right-join to get an inner
+join.  If there are no columns beyond the join columns, then
+this is equivalent to set intersection.
+        The output file will contain records from the left
+        input file and from the right input file for which
+        the join column value match.
 
 By default, node files are joined on the id column, while edge files are joined
 on the node1 column. The label and node2 columns may be added to the edge file
@@ -77,6 +94,10 @@ def add_arguments_extended(parser: KGTKArgumentParser, parsed_shared_args: Names
 
     parser.add_output_file()
 
+    parser.add_argument(      "--join-on-id", dest="join_on_id",
+                              help="If both input files are edge files, include the id column in the join (default=%(default)s).",
+                              type=optional_bool, nargs='?', const=True, default=False)
+
     parser.add_argument(      "--join-on-label", dest="join_on_label",
                               help="If both input files are edge files, include the label column in the join (default=%(default)s).",
                               type=optional_bool, nargs='?', const=True, default=False)
@@ -117,6 +138,7 @@ def run(left_file: KGTKFiles,
         right_file: KGTKFiles,
         output_file: KGTKFiles,
 
+        join_on_id: bool = False,
         join_on_label: bool = False,
         join_on_node2: bool = False,
 
@@ -185,6 +207,7 @@ def run(left_file: KGTKFiles,
         
         print("--left-join=%s" % str(left_join), file=error_file)
         print("--right-join=%s" % str(right_join), file=error_file)
+        print("--join-on-id=%s" % str(join_on_id), file=error_file)
         print("--join-on-label=%s" % str(join_on_label), file=error_file)
         print("--join-on-node2=%s" % str(join_on_node2), file=error_file)
         if left_join_columns is not None:
@@ -208,6 +231,7 @@ def run(left_file: KGTKFiles,
             output_path=output_file_path,
             left_join=left_join,
             right_join=right_join,
+            join_on_id=join_on_id,
             join_on_label=join_on_label,
             join_on_node2=join_on_node2,
             left_join_columns=left_join_columns,
