@@ -79,6 +79,9 @@ def add_arguments_extended(parser: KGTKArgumentParser, parsed_shared_args: Names
                               help=h("The suffix used for newly created output columns. (default=%(default)s)."),
                               default=KgtkLift.DEFAULT_OUTPUT_LIFTED_COLUMN_SUFFIX)
 
+    parser.add_argument(      "--default-value", dest="default_value",
+                              help="The value to use if a lifted label is not found. (default=%(default)s)", default="")
+
     parser.add_argument(      "--update-select-value", "--target-new-label-value", dest="output_select_column_value",
                               help=h("A new value for the select (label) column for records that received lifted values. " +
                               "The default is not to update the select(label) column."), default=None)
@@ -143,6 +146,15 @@ def add_arguments_extended(parser: KGTKArgumentParser, parsed_shared_args: Names
                               metavar="True/False",
                               type=optional_bool, nargs='?', const=True, default=False)
 
+    parser.add_argument(      "--clear-before-lift", dest="clear_before_lift",
+                              help="If true, set columns to write to the default value before lifting. (default=%(default)s).",
+                              type=optional_bool, nargs='?', const=True, default=False)
+
+    parser.add_argument(      "--overwrite", dest="overwrite",
+                              help="If true, overwrite non-default values in the columns to write. " +
+                              "If false, do not overwrite non-default values in the columns to write. (default=%(default)s).",
+                              type=optional_bool, nargs='?', const=True, default=True)
+
     KgtkReader.add_debug_arguments(parser, expert=_expert)
     # TODO: seperate reader_options for the label file.
     KgtkReaderOptions.add_arguments(parser, mode_options=True, expert=_expert)
@@ -165,6 +177,8 @@ def run(input_file: KGTKFiles,
         label_match_column_name: typing.Optional[str],
         label_value_column_name: typing.Optional[str],
 
+        default_value: str,
+
         remove_label_records: bool = False,
         sort_lifted_labels: bool = True,
         suppress_duplicate_labels: bool = True,
@@ -174,6 +188,9 @@ def run(input_file: KGTKFiles,
         input_is_presorted: bool = False,
         labels_are_presorted: bool = False,
 
+        clear_before_lift: bool = False,
+        overwrite: bool = False,
+
         errors_to_stdout: bool = False,
         errors_to_stderr: bool = True,
         show_options: bool = False,
@@ -232,14 +249,17 @@ def run(input_file: KGTKFiles,
         if label_value_column_name is not None:
             print("--label-value-column=%s" % label_value_column_name, file=error_file, flush=True)
 
-        print("--remove-label-records=%s" % str(remove_label_records))
-        print("--sort-lifted-labels=%s" % str(sort_lifted_labels))
-        print("--suppress-duplicate-labels=%s" % str(suppress_duplicate_labels))
-        print("--suppress-empty-columns=%s" % str(suppress_empty_columns))
-        print("--ok-if-no-labels=%s" % str(ok_if_no_labels))
-        print("--prefilter-labels=%s" % str(prefilter_labels))
-        print("--input-file-is-presorted=%s" % str(input_is_presorted))
-        print("--label-file-is-presorted=%s" % str(labels_are_presorted))
+        print("--default_value=%s" % repr(remove_label_records))
+        print("--remove-label-records=%s" % repr(remove_label_records))
+        print("--sort-lifted-labels=%s" % repr(sort_lifted_labels))
+        print("--suppress-duplicate-labels=%s" % repr(suppress_duplicate_labels))
+        print("--suppress-empty-columns=%s" % repr(suppress_empty_columns))
+        print("--ok-if-no-labels=%s" % repr(ok_if_no_labels))
+        print("--prefilter-labels=%s" % repr(prefilter_labels))
+        print("--input-file-is-presorted=%s" % repr(input_is_presorted))
+        print("--label-file-is-presorted=%s" % repr(labels_are_presorted))
+        print("--clear-before-lift=%s" % repr(clear_before_lift))
+        print("--overwrite=%s" % repr(overwrite))
         reader_options.show(out=error_file)
         value_options.show(out=error_file)
         print("=======", file=error_file, flush=True)
@@ -263,6 +283,8 @@ def run(input_file: KGTKFiles,
             label_match_column_name=label_match_column_name,
             label_value_column_name=label_value_column_name,
 
+            default_value=default_value,
+
             remove_label_records=remove_label_records,
             sort_lifted_labels=sort_lifted_labels,
             suppress_duplicate_labels=suppress_duplicate_labels,
@@ -271,6 +293,10 @@ def run(input_file: KGTKFiles,
             prefilter_labels=prefilter_labels,
             input_is_presorted=input_is_presorted,
             labels_are_presorted=labels_are_presorted,
+
+            clear_before_lift=clear_before_lift,
+            overwrite=overwrite,
+
             reader_options=reader_options,
             value_options=value_options,
             error_file=error_file,
