@@ -18,12 +18,8 @@ TODO: Add support for alternative envelope formats, such as JSON.
 
 from argparse import ArgumentParser, _ArgumentGroup, Namespace, SUPPRESS
 import attr
-import bz2
 from enum import Enum
-import gzip
 import io
-import lz4 # type: ignore
-import lzma
 from multiprocessing import Process, Queue
 from pathlib import Path
 import sys
@@ -52,6 +48,7 @@ class KgtkReaderMode(Enum):
 class KgtkReaderOptions():
     ERROR_LIMIT_DEFAULT: int = 1000
     GZIP_QUEUE_SIZE_DEFAULT: int = GunzipProcess.GZIP_QUEUE_SIZE_DEFAULT
+    MGZIP_THREAD_COUNT_DEFAULT: int = 3
 
     # TODO: use an enum
     INPUT_FORMAT_CSV: str = "csv"
@@ -119,9 +116,13 @@ class KgtkReaderOptions():
     # Other implementation options?
     input_format: typing.Optional[str] = attr.ib(validator=attr.validators.optional(attr.validators.instance_of(str)), default=None) # TODO: use an Enum
     compression_type: typing.Optional[str] = attr.ib(validator=attr.validators.optional(attr.validators.instance_of(str)), default=None) # TODO: use an Enum
+    use_mgzip: bool = attr.ib(validator=attr.validators.instance_of(bool), default=False)
+    mgzip_threads: int = attr.ib(validator=attr.validators.instance_of(int), default=MGZIP_THREAD_COUNT_DEFAULT)
     gzip_in_parallel: bool = attr.ib(validator=attr.validators.instance_of(bool), default=False)
     gzip_queue_size: int = attr.ib(validator=attr.validators.instance_of(int), default=GZIP_QUEUE_SIZE_DEFAULT)
 
+    prohibit_whitespace_in_column_names: bool = attr.ib(validator=attr.validators.instance_of(bool), default=False)
+
     @classmethod
     def add_arguments(cls,
                       parser: ArgumentParser,
@@ -185,15 +186,25 @@ class KgtkReaderOptions():
                             help=h(prefix3 + "The maximum number of errors to report before failing (default=%(default)s)"),
                             type=int, **d(default=cls.ERROR_LIMIT_DEFAULT))
 
+        fgroup.add_argument(prefix1 + "use-mgzip",
+                            dest=prefix2 + "use_mgzip",
+                            metavar="optional True|False",
+                            help=h(prefix3 + "Execute multithreaded gzip. (default=%(default)s)."),
+                            type=optional_bool, nargs='?', const=True, **d(default=False))
+
+        fgroup.add_argument(prefix1 + "mgzip-threads", dest=prefix2 + "mgzip_threads",
+                            help=h(prefix3 + "Multithreaded gzip thread  count. (default=%(default)s)."),
+                            type=int, **d(default=cls.MGZIP_THREAD_COUNT_DEFAULT))
+
         fgroup.add_argument(prefix1 + "gzip-in-parallel",
                             dest=prefix2 + "gzip_in_parallel",
                             metavar="optional True|False",
-                            help=h(prefix3 + "Execute gzip in parallel (default=%(default)s)."),
+                            help=h(prefix3 + "Execute gzip in parallel. (default=%(default)s)."),
                             type=optional_bool, nargs='?', const=True, **d(default=False))
 
         fgroup.add_argument(prefix1 + "gzip-queue-size",
                             dest=prefix2 + "gzip_queue_size",
-                            help=h(prefix3 + "Queue size for parallel gzip (default=%(default)s)."),
+                            help=h(prefix3 + "Queue size for parallel gzip. (default=%(default)s)."),
                             type=int, **d(default=cls.GZIP_QUEUE_SIZE_DEFAULT))
 
         if mode_options:
@@ -226,6 +237,12 @@ class KgtkReaderOptions():
                             help=h(prefix3 + "The action to take when a column name is unsafe (default=%(default)s)."),
                             type=ValidationAction, action=EnumNameAction, **d(default=ValidationAction.REPORT))
 
+        fgroup.add_argument(prefix1 + "prohibit-whitespace-in-column-names",
+                            dest=prefix2 + "prohibit_whitespace_in_column_names",
+                            metavar="optional True|False",
+                            help=h(prefix3 + "Prohibit whitespace in column names. (default=%(default)s)."),
+                            type=optional_bool, nargs='?', const=True, **d(default=False))
+
         sgroup: _ArgumentGroup = parser.add_argument_group(h(prefix3 + "Pre-validation sampling"),
                                                            h("Options affecting " + prefix4 + "pre-validation data line sampling."))
         
@@ -354,6 +371,8 @@ class KgtkReaderOptions():
             every_nth_record=lookup("every_nth_record", 1),
             fill_short_lines=lookup("fill_short_lines", False),
             force_column_names=lookup("force_column_names", None),
+            use_mgzip=lookup("use_mgzip", False),
+            mgzip_threads=lookup("mgzip_threads", cls.MGZIP_THREAD_COUNT_DEFAULT),
             gzip_in_parallel=lookup("gzip_in_parallel", False),
             gzip_queue_size=lookup("gzip_queue_size", KgtkReaderOptions.GZIP_QUEUE_SIZE_DEFAULT),
             header_error_action=lookup("header_error_action", ValidationAction.EXCLUDE),
@@ -366,11 +385,12 @@ class KgtkReaderOptions():
             repair_and_validate_lines=lookup("repair_and_validate_lines", False),
             repair_and_validate_values=lookup("repair_and_validate_values", False),
             short_line_action=lookup("short_line_action", ValidationAction.EXCLUDE),
-            skip_header_record=lookup("skip_header_recordb", False),
+            skip_header_record=lookup("skip_header_record", False),
             tail_count=lookup("tail_count", None),
             truncate_long_lines=lookup("truncate_long_lines", False),
             unsafe_column_name_action=lookup("unsafe_column_name_action", ValidationAction.REPORT),
             whitespace_line_action=lookup("whitespace_line_action", ValidationAction.EXCLUDE),
+            prohibit_whitespace_in_column_names=lookup("prohibit_whitespace_in_column_names", False)
         )
 
     # Build the value parsing option structure.
@@ -416,8 +436,11 @@ class KgtkReaderOptions():
             print("%sinput-format=%s" % (prefix, str(self.input_format)), file=out)
         if self.compression_type is not None:
             print("%scompression-type=%s" % (prefix, str(self.compression_type)), file=out)
+        print("%suse-mgzip=%s" % (prefix, str(self.use_mgzip)), file=out)
+        print("%smgzip-threads=%s" % (prefix, str(self.mgzip_threads)), file=out)
         print("%sgzip-in-parallel=%s" % (prefix, str(self.gzip_in_parallel)), file=out)
         print("%sgzip-queue-size=%s" % (prefix, str(self.gzip_queue_size)), file=out)
+        print("%sprohibit-whitespace-in-column-names=%s" % (prefix, str(self.prohibit_whitespace_in_column_names)), file=out)
               
 
 DEFAULT_KGTK_READER_OPTIONS: KgtkReaderOptions = KgtkReaderOptions()
@@ -461,6 +484,13 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
     data_lines_skipped: int = attr.ib(validator=attr.validators.instance_of(int), default=0)
     data_lines_passed: int = attr.ib(validator=attr.validators.instance_of(int), default=0)
     data_lines_ignored: int = attr.ib(validator=attr.validators.instance_of(int), default=0)
+    data_lines_filled: int = attr.ib(validator=attr.validators.instance_of(int), default=0)
+    data_lines_truncated: int = attr.ib(validator=attr.validators.instance_of(int), default=0)
+    data_lines_excluded_short: int = attr.ib(validator=attr.validators.instance_of(int), default=0)
+    data_lines_excluded_long: int = attr.ib(validator=attr.validators.instance_of(int), default=0)
+    data_lines_excluded_blank_fields: int = attr.ib(validator=attr.validators.instance_of(int), default=0)
+    data_lines_excluded_invalid_values: int = attr.ib(validator=attr.validators.instance_of(int), default=0)
+    data_lines_excluded_prohibited_lists: int = attr.ib(validator=attr.validators.instance_of(int), default=0)
     data_errors_reported: int = attr.ib(validator=attr.validators.instance_of(int), default=0)
 
     # Is this an edge file or a node file?
@@ -520,7 +550,8 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
                                header_line=header,
                                who=who,
                                error_action=options.unsafe_column_name_action,
-                               error_file=error_file)
+                               error_file=error_file,
+                               prohibit_whitespace_in_column_names=options.prohibit_whitespace_in_column_names)
 
         # Build a map from column name to column index.
         column_name_map: typing.Mapping[str, int] = cls.build_column_name_map(column_names,
@@ -593,7 +624,7 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
             cls = EdgeReader
         
         elif is_node_file:
-            # We'll instantiate an NodeReader, which is a subclass of KgtkReader.
+            # We'll instantiate a NodeReader, which is a subclass of KgtkReader.
             # The NodeReader import is deferred to avoid circular imports.
             from kgtk.io.nodereader import NodeReader
             
@@ -629,29 +660,45 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
                               file_name: str,
                               file_or_path: typing.Union[Path, typing.TextIO],
                               who: str,
+                              use_mgzip: bool,
+                              mgzip_threads: int,
                               error_file: typing.TextIO,
                               verbose: bool)->typing.TextIO:
         
         # TODO: find a better way to coerce typing.IO[Any] to typing.TextIO
         if compression_type in [".gz", "gz"]:
-            if verbose:
-                print("%s: reading gzip %s" % (who, file_name), file=error_file, flush=True)
-            return gzip.open(file_or_path, mode="rt") # type: ignore
+            if use_mgzip:
+                if verbose:
+                    print("%s: reading gzip with %d threads: %s" % (who, mgzip_threads, file_name), file=error_file, flush=True)
+                import mgzip
+                if isinstance(file_or_path, Path):
+                    return mgzip.open(str(file_or_path), mode="rt", thread=mgzip_threads) # type: ignore
+                else:
+                    return mgzip.open(file_or_path, mode="rt", thread=mgzip_threads) # type: ignore
+            else:
+                if verbose:
+                    print("%s: reading gzip %s" % (who, file_name), file=error_file, flush=True)
+                import gzip
+                return gzip.open(file_or_path, mode="rt") # type: ignore
         
         elif compression_type in [".bz2", "bz2"]:
             if verbose:
                 print("%s: reading bz2 %s" % (who, file_name), file=error_file, flush=True)
+            import bz2
             return bz2.open(file_or_path, mode="rt") # type: ignore
         
         elif compression_type in [".xz", "xz"]:
             if verbose:
                 print("%s: reading lzma %s" % (who, file_name), file=error_file, flush=True)
+            import lzma
             return lzma.open(file_or_path, mode="rt") # type: ignore
         
         elif compression_type in [".lz4", "lz4"]:
             if verbose:
                 print("%s: reading lz4 %s" % (who, file_name), file=error_file, flush=True)
+            import lz4 # type: ignore
             return lz4.frame.open(file_or_path, mode="rt") # type: ignore
+
         else:
             # TODO: throw a better exception.
                 raise ValueError("%s: Unexpected compression_type '%s'" % (who, compression_type))
@@ -665,13 +712,21 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
         who: str = cls.__name__
         if file_path is None or str(file_path) == "-":
             if options.compression_type is not None and len(options.compression_type) > 0:
-                return ClosableIterTextIOWrapper(cls._open_compressed_file(options.compression_type, "-", sys.stdin, who, error_file, verbose))
+                return ClosableIterTextIOWrapper(cls._open_compressed_file(options.compression_type,
+                                                                           "-",
+                                                                           sys.stdin,
+                                                                           who,
+                                                                           options.use_mgzip,
+                                                                           options.mgzip_threads,
+                                                                           error_file,
+                                                                           verbose))
             else:
                 if verbose:
                     print("%s: reading stdin" % who, file=error_file, flush=True)
                 return ClosableIterTextIOWrapper(sys.stdin)
 
         if str(file_path).startswith("<"):
+            # Note: compression is not currently supported for fd input files.
             fd: int = int(str(file_path)[1:])
             if verbose:
                 print("%s: reading file descriptor %d" % (who, fd), file=error_file, flush=True)
@@ -680,22 +735,36 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
         if verbose:
             print("%s: File_path.suffix: %s" % (who, file_path.suffix), file=error_file, flush=True)
 
-        gzip_file: typing.TextIO
+        input_file: typing.TextIO
         if options.compression_type is not None and len(options.compression_type) > 0:
-            gzip_file = cls._open_compressed_file(options.compression_type, str(file_path), file_path, who, error_file, verbose)
+            input_file = cls._open_compressed_file(options.compression_type,
+                                                   str(file_path),
+                                                   file_path,
+                                                   who,
+                                                   options.use_mgzip,
+                                                   options.mgzip_threads,
+                                                   error_file,
+                                                   verbose)
         elif file_path.suffix in [".bz2", ".gz", ".lz4", ".xz"]:
-            gzip_file = cls._open_compressed_file(file_path.suffix, str(file_path), file_path, who, error_file, verbose)
+            input_file = cls._open_compressed_file(file_path.suffix,
+                                                   str(file_path),
+                                                   file_path,
+                                                   who,
+                                                   options.use_mgzip,
+                                                   options.mgzip_threads,
+                                                   error_file,
+                                                   verbose)
         else:
             if verbose:
                 print("%s: reading file %s" % (who, str(file_path)), file=error_file, flush=True)
             return ClosableIterTextIOWrapper(open(file_path, "r"))
 
         if options.gzip_in_parallel:
-            gzip_thread: GunzipProcess = GunzipProcess(gzip_file, Queue(options.gzip_queue_size))
+            gzip_thread: GunzipProcess = GunzipProcess(input_file, Queue(options.gzip_queue_size))
             gzip_thread.start()
             return gzip_thread
         else:
-            return ClosableIterTextIOWrapper(gzip_file)
+            return ClosableIterTextIOWrapper(input_file)
             
 
     @classmethod
@@ -720,14 +789,27 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
             if verbose:
                 print("header: %s" % header, file=error_file, flush=True)
 
+            input_format: str
+            if options.input_format is None:
+                input_format = KgtkReaderOptions.INPUT_FORMAT_KGTK
+            else:
+                input_format = options.input_format
+            if verbose:
+                print("input format: %s" % input_format, file=error_file, flush=True)
+
             # Split the first line into column names.
-            #
-            # TODO: if options.input_format == KgtkReaderOptions.INPUT_FORMAT_CSV, be smarter.
-            return header, header.split(options.column_separator)
+            if input_format == KgtkReaderOptions.INPUT_FORMAT_CSV:
+                return header, cls.csvsplit(header)
+            else:
+                return header, header.split(options.column_separator)
         else:
             # Skip the first record to override the column names in the file.
             # Do not skip the first record if the file does not hae a header record.
+            if verbose:
+                print("Forcing column names", file=error_file, flush=True)
             if options.skip_header_record:
+                if verbose:
+                    print("Skipping a header record", file=error_file, flush=True)
                 try:
                     next(source)
                 except StopIteration:
@@ -782,7 +864,8 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
         else:
             print("%s" % line, file=self.reject_file)
 
-    def csvsplit(self, line: str)->typing.List[str]:
+    @classmethod
+    def csvsplit(cls, line: str)->typing.List[str]:
         row: typing.List[str] = [ ]
         item: str = ""
         c: str
@@ -888,18 +971,21 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
                 if self.options.empty_line_action != ValidationAction.PASS and len(line) == 0:
                     if self.exclude_line(self.options.empty_line_action, "saw an empty line", line):
                         self.reject(line)
+                        self.data_lines_ignored += 1
                         continue
 
                 # Ignore comment lines:
                 if self.options.comment_line_action != ValidationAction.PASS  and line[0] == self.COMMENT_INDICATOR:
                     if self.exclude_line(self.options.comment_line_action, "saw a comment line", line):
                         self.reject(line)
+                        self.data_lines_ignored += 1
                         continue
 
                 # Ignore whitespace lines
                 if self.options.whitespace_line_action != ValidationAction.PASS and line.isspace():
                     if self.exclude_line(self.options.whitespace_line_action, "saw a whitespace line", line):
                         self.reject(line)
+                        self.data_lines_ignored += 1
                         continue
 
             if input_format == KgtkReaderOptions.INPUT_FORMAT_CSV:
@@ -912,10 +998,12 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
                 if self.options.fill_short_lines and len(row) < self.column_count:
                     while len(row) < self.column_count:
                         row.append("")
+                    self.data_lines_filled += 1
                     
                 # Optionally remove extra trailing columns:
                 if self.options.truncate_long_lines and len(row) > self.column_count:
                     row = row[:self.column_count]
+                    self.data_lines_truncated += 1
                             
                 # Optionally validate that the line contained the right number of columns:
                 #
@@ -927,6 +1015,7 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
                                                                                 line),
                                          line):
                         self.reject(line)
+                        self.data_lines_excluded_short += 1
                         continue
                              
                 if self.options.long_line_action != ValidationAction.PASS and len(row) > self.column_count:
@@ -937,10 +1026,12 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
                                                                                            line),
                                          line):
                         self.reject(line)
+                        self.data_lines_excluded_long += 1
                         continue
 
-                if self._ignore_if_blank_fields(row, line):
+                if self._ignore_if_blank_required_fields(row, line):
                     self.reject(line)
+                    self.data_lines_excluded_blank_fields += 1
                     continue
 
             if repair_and_validate_values:
@@ -950,11 +1041,13 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
                     # and iterator methods below.
                     if self._ignore_invalid_values(row, line):
                         self.reject(line)
+                        self.data_lines_excluded_invalid_values += 1
                         continue
 
                 if self.options.prohibited_list_action != ValidationAction.PASS:
                     if self._ignore_prohibited_lists(row, line):
                         self.reject(line)
+                        self.data_lines_excluded_prohibited_lists += 1
                         continue
 
             self.data_lines_passed += 1
@@ -1163,26 +1256,23 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
         Returns True to indicate that the row should be ignored (skipped).
 
         """
-        error_file: typing.Optional[io.StringIO] = None
         problems: typing.List[str] = [ ] # Build a list of problems.
         idx: int
         item: str
         for idx, item in enumerate(row):
             if len(item) > 0: # Optimize the common case of empty columns.
-                if self.verbose:
-                    error_file = io.StringIO()
-                kv: KgtkValue = KgtkValue(item, options=self.value_options, error_file=self.error_file, verbose=self.verbose)
+                error_buffer: io.StringIO = io.StringIO()
+                kv: KgtkValue = KgtkValue(item, options=self.value_options, error_file=error_buffer, verbose=self.verbose)
                 if not kv.is_valid():
-                    if error_file is not None:
-                        problems.append("col %d (%s) value %s: %s" % (idx, self.column_names[idx], repr(item), error_file.getvalue().rstrip()))
+                    if self.verbose:
+                        problems.append("col %d (%s) value %s: %s" % (idx, self.column_names[idx], repr(item), error_buffer.getvalue().rstrip()))
                     problems.append("col %d (%s) value %s is an %s" % (idx, self.column_names[idx], repr(item), kv.describe()))
                 if kv.repaired:
                     # If this value was repaired, update the item in the row.
                     #
                     # Warning: We expect this change to be seen by the caller.
                     row[idx] = kv.value
-                if error_file is not None:
-                    error_file.close()
+                error_buffer.close()
 
         if len(problems) == 0:
             return False
@@ -1206,25 +1296,13 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
             return
         problems.append("col %d (%s) value '%s'is a prohibited list" % (idx, self.column_names[idx], item))
 
-    def _ignore_prohibited_lists(self, row: typing.List[str], line: str)->bool:
-        """
-        KGTK File Format v2 prohibits "|" lists in the node1, label, and node2 columns.
-        """
-        problems: typing.List[str] = [ ] # Build a list of problems.
-
-        self._ignore_prohibited_list(self.node1_column_idx, row, line, problems)
-        self._ignore_prohibited_list(self.label_column_idx, row, line, problems)
-        self._ignore_prohibited_list(self.node2_column_idx, row, line, problems)
-
-        if len(problems) == 0:
-            return False
-
-        return self.exclude_line(self.options.invalid_value_action,
-                                 "\n".join(problems),
-                                 line)
 
     # May be overridden
-    def _ignore_if_blank_fields(self, values: typing.List[str], line: str)->bool:
+    def _ignore_prohibited_lists(self, row: typing.List[str], line: str)->bool:
+        return False
+
+    # May be overridden
+    def _ignore_if_blank_required_fields(self, values: typing.List[str], line: str)->bool:
         return False
 
     # May be overridden
@@ -1256,20 +1334,43 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
 
         return merged_columns
 
-    def get_node1_column_index(self, column_name: typing.Optional[str] = None)->int:
+    def _get_special_column_index(self, column_name_or_idx: typing.Optional[str], special_column_idx: int)->int:
+        """
+        Get the special column index, unless an overriding column
+        index or name name is provided.  Returns -1 if no column found.
+
+        Note: column index values start with 1 for the first column.
+        """
+        if column_name_or_idx is None or len(column_name_or_idx) == 0:
+            return special_column_idx
+        elif column_name_or_idx in self.column_name_map:
+            # By performing this test before the isdigit() test, we give
+            # precedence to columns with names that are integers.
+            return self.column_name_map[column_name_or_idx]
+        elif column_name_or_idx.isdigit():
+            column_idx: int = int(column_name_or_idx) - 1
+            if column_idx < 0:
+                return -1
+            if column_idx >= self.column_count:
+                return -1
+            else:
+                return column_idx
+        else:
+            return -1
+
+    def get_node1_column_index(self, column_name_or_idx: typing.Optional[str] = None)->int:
         """
         Get the node1 column index, unless an overriding column
-        name is provided.  Returns -1 if no column found.
+        index or name is provided.  Returns -1 if no column found.
         """
-        if column_name is None or len(column_name) == 0:
-            return self.node1_column_idx
-        else:
-            return self.column_name_map.get(column_name, -1)
+        return self._get_special_column_index(column_name_or_idx, self.node1_column_idx)
 
     def get_node1_canonical_name(self, column_name: typing.Optional[str]=None)->str:
         """
         Get the canonical name for the node1 column, unless an
         overriding name is provided.
+
+        TODO: Should we convert column indexes to column names?
         """
         if column_name is not None and len(column_name) > 0:
             return column_name
@@ -1287,15 +1388,12 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
         else:
             return ""
             
-    def get_label_column_index(self, column_name: typing.Optional[str] = None)->int:
+    def get_label_column_index(self, column_name_or_idx: typing.Optional[str] = None)->int:
         """
         Get the label column index, unless an overriding column
-        name is provided.  Returns -1 if no column found.
+        index or name is provided.  Returns -1 if no column found.
         """
-        if column_name is None or len(column_name) == 0:
-            return self.label_column_idx
-        else:
-            return self.column_name_map.get(column_name, -1)
+        return self._get_special_column_index(column_name_or_idx, self.label_column_idx)
             
     def get_label_canonical_name(self, column_name: typing.Optional[str]=None)->str:
         """
@@ -1318,15 +1416,12 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
         else:
             return ""
 
-    def get_node2_column_index(self, column_name: typing.Optional[str] = None)->int:
+    def get_node2_column_index(self, column_name_or_idx: typing.Optional[str] = None)->int:
         """
         Get the node2 column index, unless an overriding column
-        name is provided.  Returns -1 if no column found.
+        index or name is provided.  Returns -1 if no column found.
         """
-        if column_name is None or len(column_name) == 0:
-            return self.node2_column_idx
-        else:
-            return self.column_name_map.get(column_name, -1)
+        return self._get_special_column_index(column_name_or_idx, self.node2_column_idx)
             
 
     def get_node2_canonical_name(self, column_name: typing.Optional[str]=None)->str:
@@ -1350,15 +1445,12 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
         else:
             return ""
             
-    def get_id_column_index(self, column_name: typing.Optional[str] = None)->int:
+    def get_id_column_index(self, column_name_or_idx: typing.Optional[str] = None)->int:
         """
         Get the id column index, unless an overriding column
-        name is provided.  Returns -1 if no column found.
+        index or name is provided.  Returns -1 if no column found.
         """
-        if column_name is None or len(column_name) == 0:
-            return self.id_column_idx
-        else:
-            return self.column_name_map.get(column_name, -1)
+        return self._get_special_column_index(column_name_or_idx, self.id_column_idx)
             
     def get_id_canonical_name(self, column_name: typing.Optional[str]=None)->str:
         """
@@ -1406,27 +1498,77 @@ class KgtkReader(KgtkBase, ClosableIter[typing.List[str]]):
         # or choices.  That will avoid the argparse bug, too.
         if expert:
             errors_to = egroup.add_mutually_exclusive_group()
-            errors_to.add_argument(      "--errors-to-stdout", dest="errors_to_stdout",
-                                         help="Send errors to stdout instead of stderr",
-                                         action="store_true")
-            errors_to.add_argument(      "--errors-to-stderr", dest="errors_to_stderr",
-                                         help="Send errors to stderr instead of stdout",
-                                         action="store_true")
+            errors_to.add_argument(      "--errors-to-stdout", dest="errors_to_stdout", metavar="optional True|False",
+                                         help="Send errors to stdout instead of stderr. (default=%(default)s).",
+                                         type=optional_bool, nargs='?', const=True, default=False)
+
+            errors_to.add_argument(      "--errors-to-stderr", dest="errors_to_stderr", metavar="optional True|False",
+                                         help="Send errors to stderr instead of stdout. (default=%(default)s).",
+                                         type=optional_bool, nargs='?', const=True, default=False)
         else:
-            egroup.add_argument(      "--errors-to-stderr", dest="errors_to_stderr",
-                                      help=h("Send errors to stderr instead of stdout"),
-                                      action="store_true")
-            egroup.add_argument(      "--errors-to-stdout", dest="errors_to_stdout",
-                                      help=h("Send errors to stdout instead of stderr"),
-                                      action="store_true")
+            egroup.add_argument(      "--errors-to-stderr", dest="errors_to_stderr", metavar="optional True|False",
+                                      help=h("Send errors to stderr instead of stdout. (default=%(default)s),"),
+                                      type=optional_bool, nargs='?', const=True, default=False)
 
-        egroup.add_argument(      "--show-options", dest="show_options", help=h("Print the options selected (default=%(default)s)."), action='store_true')
+            egroup.add_argument(      "--errors-to-stdout", dest="errors_to_stdout", metavar="optional True|False",
+                                      help=h("Send errors to stdout instead of stderr. (default=%(default)s)."),
+                                      type=optional_bool, nargs='?', const=True, default=False)
 
-        egroup.add_argument("-v", "--verbose", dest="verbose", help="Print additional progress messages (default=%(default)s).", action='store_true')
+        egroup.add_argument(      "--show-options", dest="show_options", metavar="optional True|False",
+                                  help=h("Print the options selected (default=%(default)s)."),
+                                  type=optional_bool, nargs='?', const=True, default=False)
 
-        egroup.add_argument(      "--very-verbose", dest="very_verbose",
+        egroup.add_argument("-v", "--verbose", dest="verbose", metavar="optional True|False",
+                            help="Print additional progress messages (default=%(default)s).",
+                            type=optional_bool, nargs='?', const=True, default=False)
+
+        egroup.add_argument(      "--very-verbose", dest="very_verbose", metavar="optional True|False",
                                   help=h("Print additional progress messages (default=%(default)s)."),
-                                  action='store_true')
+                                  type=optional_bool, nargs='?', const=True, default=False)
+
+    @classmethod
+    def show_debug_arguments(cls,
+                             errors_to_stdout: bool = False,
+                             errors_to_stderr: bool = True,
+                             show_options: bool = False,
+                             verbose: bool = False,
+                             very_verbose: bool = False,
+                             out: typing.TextIO = sys.stderr):
+        if errors_to_stdout:
+            print("--errors-to-stdout", file=out)
+        if errors_to_stderr:
+            print("--errors-to-stderr", file=out)
+        if show_options:
+            print("--show-options", file=out)
+        if verbose:
+            print("--verbose", file=out)
+        if very_verbose:
+            print("--very-verbose", file=out)
+
+    def report_summary(self):
+        print("Data lines read: %d" % self.data_lines_read, file=self.error_file, flush=True)
+        print("Data lines passed: %d" % self.data_lines_passed, file=self.error_file, flush=True)
+        if self.data_lines_skipped > 0:
+            print("Data lines skipped: %d" % self.data_lines_skipped, file=self.error_file, flush=True)
+        if self.data_lines_ignored > 0:
+            print("Data lines ignored: %d" % self.data_lines_ignored, file=self.error_file, flush=True)
+        if self.data_lines_filled > 0:
+            print("Data lines filled: %d" % self.data_lines_filled, file=self.error_file, flush=True)
+        if self.data_lines_truncated > 0:
+            print("Data lines truncated: %d" % self.data_lines_truncated, file=self.error_file, flush=True)
+        if self.data_lines_excluded_short > 0:
+            print("Data lines excluded due to too few columns: %d" % self.data_lines_excluded_short, file=self.error_file, flush=True)
+        if self.data_lines_excluded_long > 0:
+            print("Data lines excluded due to too many columns: %d" % self.data_lines_excluded_long, file=self.error_file, flush=True)
+        if self.data_lines_excluded_blank_fields > 0:
+            print("Data lines excluded due to blank fields: %d" % self.data_lines_excluded_blank_fields, file=self.error_file, flush=True)
+        if self.data_lines_excluded_invalid_values > 0:
+            print("Data lines excluded due to invalid values: %d" % self.data_lines_excluded_invalid_values, file=self.error_file, flush=True)
+        if self.data_lines_excluded_prohibited_lists > 0:
+            print("Data lines excluded due to prohibited lists: %d" % self.data_lines_excluded_prohibited_lists, file=self.error_file, flush=True)
+        if self.data_errors_reported > 0:
+            print("Data errors reported: %d" % self.data_errors_reported, file=self.error_file, flush=True)
+
         
 def main():
     """
