@@ -31,8 +31,9 @@ from kgtk.value.kgtkvalueoptions import KgtkValueOptions
 
 @attr.s(slots=True, frozen=True)
 class KgtkLift(KgtkFormat):
-    DEFAULT_OUTPUT_LIFTED_COLUMN_SUFFIX: str = ";label"
+    DEFAULT_OUTPUT_LIFTED_COLUMN_SEPARATOR: str = ";"
     DEFAULT_LABEL_SELECT_COLUMN_VALUE: str = "label"
+    DEFAULT_OUTPUT_LIFTED_COLUMN_SUFFIX: str = DEFAULT_OUTPUT_LIFTED_COLUMN_SEPARATOR + DEFAULT_LABEL_SELECT_COLUMN_VALUE
 
     input_file_path: Path = attr.ib(validator=attr.validators.instance_of(Path))
     label_file_path: typing.Optional[Path] = attr.ib(validator=attr.validators.optional(attr.validators.instance_of(Path)))
@@ -70,6 +71,11 @@ class KgtkLift(KgtkFormat):
     input_is_presorted: bool = attr.ib(validator=attr.validators.instance_of(bool), default=False)
     labels_are_presorted: bool = attr.ib(validator=attr.validators.instance_of(bool), default=False)
 
+    default_value: str = attr.ib(validator=attr.validators.instance_of(str), default="")
+
+    clear_before_lift: bool = attr.ib(validator=attr.validators.instance_of(bool), default=False)
+    overwrite: bool = attr.ib(validator=attr.validators.instance_of(bool), default=True)
+
     # TODO: add rewind logic here and KgtkReader
 
     # TODO: find working validators
@@ -228,7 +234,7 @@ class KgtkLift(KgtkFormat):
                 # This is a label definition row.
                 label_key = row[label_match_column_idx]
                 label_value: str = row[label_value_column_idx]
-                if len(label_value) > 0:
+                if label_value != self.default_value:
                     if label_key in labels:
                         # This label already exists in the table.
                         if self.suppress_duplicate_labels:
@@ -363,7 +369,7 @@ class KgtkLift(KgtkFormat):
         )->bool:
         output_row: typing.List[str] = row.copy()
         if new_columns > 0:
-            output_row.extend([""] * new_columns)
+            output_row.extend([self.default_value] * new_columns)
         output_select_column_idx: int = input_select_column_idx
                 
         do_write: bool = True
@@ -384,10 +390,14 @@ class KgtkLift(KgtkFormat):
             did_lift: bool = False
             lifted_column_idx: int
             for idx, lifted_column_idx in enumerate(lifted_column_idxs):
-                label_key: str = row[lifted_column_idx]
-                if label_key in labels:
-                    output_row[lifted_output_column_idxs[idx]] = labels[label_key]
-                    did_lift = True # What if we want to note if we lifted all columns?
+                if self.clear_before_lift:
+                    output_row[lifted_output_column_idxs[idx]] = self.default_value
+                if self.overwrite or output_row[lifted_output_column_idxs[idx]] == self.default_value:
+                    label_key: str = row[lifted_column_idx]
+                    if label_key in labels:
+                        label_value: str = labels[label_key]
+                        output_row[lifted_output_column_idxs[idx]] = label_value
+                        did_lift = True # What if we want to note if we lifted all columns?
             if did_lift and output_select_column_idx >= 0 and self.output_select_column_value is not None:
                 output_row[output_select_column_idx] = self.output_select_column_value
 
@@ -399,12 +409,14 @@ class KgtkLift(KgtkFormat):
         # Build the output column names.
         output_column_names: typing.List[str] = ikr.column_names.copy()
         lifted_output_column_idxs: typing.List[int] = [ ]
-        for idx in lifted_column_idxs:
+        lifted_idx: int
+        column_idx: int
+        for lifted_idx, column_idx in enumerate(lifted_column_idxs):
             lifted_column_name: str
-            if self.output_lifted_column_names is not None and idx < len(self.output_lifted_column_names):
-                lifted_column_name = self.output_lifted_column_names[idx]
+            if self.output_lifted_column_names is not None and lifted_idx < len(self.output_lifted_column_names):
+                lifted_column_name = self.output_lifted_column_names[lifted_idx]
             else:
-                lifted_column_name = ikr.column_names[idx] + self.output_lifted_column_suffix
+                lifted_column_name = ikr.column_names[column_idx] + self.output_lifted_column_suffix
 
             if lifted_column_name in ikr.column_name_map:
                 # Overwrite an existing lifted output column.
@@ -434,6 +446,8 @@ class KgtkLift(KgtkFormat):
                                          require_all_columns=False,
                                          prohibit_extra_columns=True,
                                          fill_missing_columns=True,
+                                         use_mgzip=False if self.reader_options is None else self.reader_options.use_mgzip , # Hack!
+                                         mgzip_threads=3 if self.reader_options is None else self.reader_options.mgzip_threads , # Hack!
                                          gzip_in_parallel=False,
                                          verbose=self.verbose,
                                          very_verbose=self.very_verbose)        
@@ -618,7 +632,7 @@ class KgtkLift(KgtkFormat):
         # iterations in case the input file has multiple records with
         # the same value to lift.
         last_value_to_lift: typing.Optional[str] = None
-        lifted_label_value: str = ""
+        lifted_label_value: str = self.default_value
 
         if self.verbose:
             print("Processing the input records.", file=self.error_file, flush=True)
@@ -629,10 +643,11 @@ class KgtkLift(KgtkFormat):
             input_line_count += 1
 
             if self.input_select_column_value is not None and input_select_column_idx >= 0:
+                # Skip input rows that do not meet the selection criterion.
                 if row[input_select_column_idx] != self.input_select_column_value:
                     if new_columns > 0:
                         output_row = row.copy()
-                        output_row.append("")
+                        output_row.append(self.default_value)
                     else:
                         output_row = row
                     ew.write(output_row)
@@ -641,7 +656,7 @@ class KgtkLift(KgtkFormat):
             value_to_lift: str = row[lift_column_idx]
             if last_value_to_lift is None or value_to_lift != last_value_to_lift:
                 last_value_to_lift = value_to_lift
-                lifted_label_value = ""
+                lifted_label_value = self.default_value
 
                 # Read label records until we come to the first record that
                 # has a node1 value equal to or greater than the value we we want to lift.
@@ -658,7 +673,7 @@ class KgtkLift(KgtkFormat):
                     if label_select_column_idx < 0 or current_label_row[label_select_column_idx] == self.label_select_column_value:
                         label_value: str = current_label_row[label_value_column_idx]
                         if len(label_value) > 0:
-                            if len(lifted_label_value) > 0:
+                            if lifted_label_value != self.default_value:
                                 if self.suppress_duplicate_labels:
                                     lifted_label_value = KgtkValue.merge_values(lifted_label_value, label_value)
                                 else:
@@ -674,11 +689,14 @@ class KgtkLift(KgtkFormat):
 
             output_row = row.copy()
             if new_columns > 0:
-                output_row.append("")
-            if len(lifted_label_value) > 0:
-                output_row[lifted_output_column_idx] = lifted_label_value
-                if self.output_select_column_value is not None and input_select_column_idx >= 0:
-                    output_row[input_select_column_idx] = self.output_select_column_value
+                output_row.append(self.default_value)
+            if self.clear_before_lift:
+                output_row[lifted_output_column_idx] = self.default_value
+            if self.overwrite or output_row[lifted_output_column_idx] != self.default_value:
+                if lifted_label_value != self.default_value:
+                    output_row[lifted_output_column_idx] = lifted_label_value
+                    if self.output_select_column_value is not None and input_select_column_idx >= 0:
+                        output_row[input_select_column_idx] = self.output_select_column_value
             ew.write(output_row)
 
         if more_labels:
@@ -687,6 +705,8 @@ class KgtkLift(KgtkFormat):
         if self.verbose:
             print("Read %d input records." % (input_line_count), file=self.error_file, flush=True)
              
+        ew.close()
+
     
     def process(self):
         # Open the input file.
@@ -767,6 +787,9 @@ def main():
                               help="The suffix used for newly created output columns. (default=%(default)s).",
                               default=KgtkLift.DEFAULT_OUTPUT_LIFTED_COLUMN_SUFFIX)
 
+    parser.add_argument(      "--default-value", dest="default_value",
+                              help="The value to use if a lifted label is not found. (default=%(default)s)", default="")
+
     parser.add_argument(      "--update-select-value", "--target-new-label-value", dest="output_select_column_value",
                               help="A new value for the select (label) column for records that received lifted values. " +
                               "The default is not to update the select(label) column.", default=None)
@@ -790,8 +813,6 @@ def main():
                               "to be lifted into the input record that is receiving lifted values. " +
                               "The default is 'node2' or its alias.", default=None)
 
-
-
     parser.add_argument(      "--remove-label-records", dest="remove_label_records",
                               help="If true, remove label records from the output. (default=%(default)s).",
                               type=optional_bool, nargs='?', const=True, default=True)
@@ -824,6 +845,15 @@ def main():
                               help="If true, the label file is presorted on the node1 column. (default=%(default)s).",
                               type=optional_bool, nargs='?', const=True, default=False)
 
+    parser.add_argument(      "--clear-before-lift", dest="clear_before_lift",
+                              help="If true, set columns to write to the default value before lifting. (default=%(default)s).",
+                              type=optional_bool, nargs='?', const=True, default=False)
+
+    parser.add_argument(      "--overwrite", dest="overwrite",
+                              help="If true, overwrite non-default values in the columns to write. " +
+                              "If false, do not overwrite non-default values in the columns to write. (default=%(default)s).",
+                              type=optional_bool, nargs='?', const=True, default=True)
+
 
     KgtkReader.add_debug_arguments(parser)
     # TODO: seperate reader options for the label file.
@@ -868,14 +898,17 @@ def main():
             print("--label-value-column=%s" % args.label_value_column_name, file=error_file, flush=True)
 
 
-        print("--remove-label-records=%s" % str(args.remove_label_records))
-        print("--sort-lifted-labels-labels=%s" % str(args.sort_lifted_labels))
-        print("--suppress-duplicate-labels=%s" % str(args.suppress_duplicate_labels))
-        print("--suppress-empty-columns=%s" % str(args.suppress_empty_columns))
-        print("--ok-if-no-labels=%s" % str(args.ok_if_no_labels))
-        print("--prefilter-labels=%s" % str(args.prefilter_labels))
-        print("--input-file-is-presorted=%s" % str(args.input_is_presorted))
-        print("--label-file-is-presorted=%s" % str(args.labels_are_presorted))
+        print("--default-value=%s" % repr(args.default_value))
+        print("--remove-label-records=%s" % repr(args.remove_label_records))
+        print("--sort-lifted-labels-labels=%s" % repr(args.sort_lifted_labels))
+        print("--suppress-duplicate-labels=%s" % repr(args.suppress_duplicate_labels))
+        print("--suppress-empty-columns=%s" % repr(args.suppress_empty_columns))
+        print("--ok-if-no-labels=%s" % repr(args.ok_if_no_labels))
+        print("--prefilter-labels=%s" % repr(args.prefilter_labels))
+        print("--input-file-is-presorted=%s" % repr(args.input_is_presorted))
+        print("--label-file-is-presorted=%s" % repr(args.labels_are_presorted))
+        print("--clear-before-lift=%s" % repr(args.clear_before_lift))
+        print("--overwrite=%s" % repr(args.overwrite))
         reader_options.show(out=error_file)
         value_options.show(out=error_file)
 
@@ -896,6 +929,8 @@ def main():
         label_select_column_value=args.label_select_column_value,
         label_match_column_name=args.label_match_column_name,
         label_value_column_name=args.label_value_column_name,
+
+        default_value=args.default_value,
         
         remove_label_records=args.remove_label_records,
         sort_lifted_labels=args.sort_lifted_labels,
@@ -905,6 +940,10 @@ def main():
         prefilter_labels=args.prefilter_labels,
         input_is_presorted=args.input_is_presorted,
         labels_are_presorted=args.labels_are_presorted,
+
+        clear_before_lift=args.clear_before_lift,
+        overwrite=args.overwrite,
+
         reader_options=reader_options,
         value_options=value_options,
         error_file=error_file,
