@@ -32,8 +32,10 @@ class KgtkCompact(KgtkFormat):
                                                                                          iterable_validator=attr.validators.instance_of(list)))
     compact_id: bool = attr.ib(validator=attr.validators.instance_of(bool), default=False)
 
-    # The field separator used in multifield joins.  The KGHT list character should be safe.
-    field_separator: str = attr.ib(validator=attr.validators.instance_of(str), default=KgtkFormat.LIST_SEPARATOR)
+    deduplicate: bool = attr.ib(validator=attr.validators.instance_of(bool), default=True)
+
+    # The field separator used in multifield joins.  The KGTK list character should be safe.
+    field_separator: str = attr.ib(validator=attr.validators.instance_of(str), default=KgtkFormat.KEY_FIELD_SEPARATOR)
 
     sorted_input: bool = attr.ib(validator=attr.validators.instance_of(bool), default=False)
     verify_sort: bool = attr.ib(validator=attr.validators.instance_of(bool), default=True)
@@ -60,7 +62,7 @@ class KgtkCompact(KgtkFormat):
     current_row: typing.Optional[typing.List[str]] = None
     current_row_lists: typing.Optional[typing.List[typing.Optional[typing.List[str]]]] = None
 
-    FIELD_SEPARATOR_DEFAULT: str = KgtkFormat.LIST_SEPARATOR
+    FIELD_SEPARATOR_DEFAULT: str = KgtkFormat.KEY_FIELD_SEPARATOR
 
     def build_key(self, row: typing.List[str], key_columns: typing.List[int])->str:
         key: str = ""
@@ -167,6 +169,7 @@ class KgtkCompact(KgtkFormat):
                 if item2 not in current_item_list:
                     current_item_list.append(item2) # Add unique items.
 
+    # TODO: Create an optimized version of this routine without the very verbose debugginng messages.
     def process_row(self,
                     input_key: str,
                     row: typing.List[str],
@@ -174,13 +177,24 @@ class KgtkCompact(KgtkFormat):
                     idb: typing.Optional[KgtkIdBuilder],
                     ew: KgtkWriter,
                     flush: bool = False):
+        if self.very_verbose:
+            print("Input key %s" % repr(input_key), file=self.error_file, flush=True)
         # Note:  This code makes the assumption that row lengths do not vary!
         if self.current_key is not None:
+            if self.very_verbose:
+                print("No current key", file=self.error_file, flush=True)
             # We have a record being built.  Write it?
             if flush or self.current_key != input_key:
+                if self.very_verbose:
+                    if flush:
+                        print("flush", file=self.error_file, flush=True)
+                    else:
+                        print("current_key %s != input_key %s" % (repr(self.current_key), repr(input_key)), file=self.error_file, flush=True)
                 # self.current_key != input_key means that the key is changing.
                 self.compact_row()
                 if self.current_row is not None:
+                    if self.very_verbose:
+                        print("writing %s" % repr(self.field_separator.join(self.current_row)), file=self.error_file, flush=True)
                     if idb is None:
                         ew.write(self.current_row)
                     else:
@@ -196,10 +210,16 @@ class KgtkCompact(KgtkFormat):
         # Are we starting a new key?
         if self.current_key is None:
             # Save the new row.
+            if self.very_verbose:
+                print("New current_key %s" % repr(self.current_key), file=self.error_file, flush=True)
             self.current_key = input_key
+            if self.very_verbose:
+                print("Expand row %s" % self.field_separator.join(row), file=self.error_file, flush=True)
             self.expand_row(row)
         else:
             # Merge into an existing row.
+            if self.very_verbose:
+                print("Merge row", file=self.error_file, flush=True)
             self.merge_row(row)
 
     def process(self):
@@ -233,35 +253,46 @@ class KgtkCompact(KgtkFormat):
 
         # Build the list of key column edges:
         key_idx_list: typing.List[int] = [ ]
-        if kr.is_edge_file:
-            # Add the KGTK edge file required columns.
-            key_idx_list.append(kr.node1_column_idx)
-            key_idx_list.append(kr.label_column_idx)
-            key_idx_list.append(kr.node2_column_idx)
-            if not self.compact_id and kr.id_column_idx >= 0:
+
+        if self.deduplicate:
+            # Use the columns in the order the appear in the input file.  If you want to
+            # use --presorted, then the input needs to be sorted the same way.
+            key_idx_list = range(kr.column_count)
+            
+        elif len(self.key_column_names) == 0:
+            if kr.is_edge_file:
+                # Add the KGTK edge file required columns.
+                key_idx_list.append(kr.node1_column_idx)
+                key_idx_list.append(kr.label_column_idx)
+                key_idx_list.append(kr.node2_column_idx)
+                if not self.compact_id and kr.id_column_idx >= 0:
+                    key_idx_list.append(kr.id_column_idx)
+
+            elif kr.is_node_file:
+                # Add the KGTK node file required column:
                 key_idx_list.append(kr.id_column_idx)
 
-        elif kr.is_node_file:
-            # Add the KGTK node file required column:
-            key_idx_list.append(kr.id_column_idx)
+            else:
+                raise ValueError("The input file is neither an edge nor a node file.  Key columns must be supplied.")
 
-        # Append additional columns to the list of key column indices,
-        # silently removing duplicates, but complaining about unknown names.
-        #
-        # TODO: warn about duplicates?
-        column_name: str
-        for column_name in self.key_column_names:
-            if column_name not in kr.column_name_map:
-                raise ValueError("Column %s is not in the input file" % (column_name))
-            key_idx: int = kr.column_name_map[column_name]
-            if key_idx not in key_idx_list:
-                key_idx_list.append(key_idx)
+        else:
+            # Append additional columns to the list of key column indices,
+            # silently removing duplicates, but complaining about unknown names.
+            #
+            # TODO: warn about duplicates?
+            column_name: str
+            for column_name in self.key_column_names:
+                if column_name not in kr.column_name_map:
+                    raise ValueError("Column %s is not in the input file" % (column_name))
+                key_idx: int = kr.column_name_map[column_name]
+                if key_idx not in key_idx_list:
+                    key_idx_list.append(key_idx)
 
         if self.verbose:
             key_idx_list_str: typing.List[str] = [ ]
             for key_idx in key_idx_list:
                 key_idx_list_str.append(str(key_idx))
-            print("key indexes: %s" % " ".join(key_idx_list_str))
+            print("key indexes: %s" % " ".join(key_idx_list_str), file=self.error_file, flush=True)
             
         # Open the output file.
         ew: KgtkWriter = KgtkWriter.open(output_column_names,
@@ -270,6 +301,8 @@ class KgtkCompact(KgtkFormat):
                                          require_all_columns=False,
                                          prohibit_extra_columns=True,
                                          fill_missing_columns=True,
+                                         use_mgzip=self.reader_options.use_mgzip, # Hack!
+                                         mgzip_threads=self.reader_options.mgzip_threads, # Hack!
                                          gzip_in_parallel=False,
                                          verbose=self.verbose,
                                          very_verbose=self.very_verbose)        
@@ -301,14 +334,18 @@ class KgtkCompact(KgtkFormat):
                             if prev_input_key < input_key:
                                 prev_input_key = input_key
                             elif prev_input_key > input_key:
-                                raise ValueError("Line %d sort violation going up: prev='%s' curr='%s'" % (input_line_count, prev_input_key, input_key))
+                                raise ValueError("Line %d sort violation going up: prev='%s' curr='%s'" % (input_line_count,
+                                                                                                           prev_input_key.replace(self.field_separator, KgtkFormat.LIST_SEPARATOR),
+                                                                                                           input_key.replace(self.field_separator, KgtkFormat.LIST_SEPARATOR)))
                             else:
                                 pass # No change in input_key
                         else:
                             if prev_input_key > input_key:
                                 prev_input_key = input_key
                             elif prev_input_key < input_key:
-                                raise ValueError("Line %d sort violation going down: prev='%s' curr='%s'" % (input_line_count, prev_input_key, input_key))
+                                raise ValueError("Line %d sort violation going down: prev='%s' curr='%s'" % (input_line_count,
+                                                                                                             prev_input_key.replace(self.field_separator, KgtkFormat.LIST_SEPARATOR),
+                                                                                                             input_key.replace(self.field_separator, KgtkFormat.LIST_SEPARATOR)))
                             else:
                                 pass # No change in input_key
                             
@@ -364,6 +401,12 @@ def main():
                               "as there are use cases that need to maintain distinct lists of secondary edges for each ID value. (default=%(default)s).",
                               type=optional_bool, nargs='?', const=True, default=False)
 
+    parser.add_argument(      "--deduplicate", dest="deduplicate",
+                              help="Treat all columns as key columns, overriding --columns and --compact-id. " +
+                              "This will remove completely duplicate records without compacting any new lists. " +
+                              "(default=%(default)s).",
+                              type=optional_bool, nargs='?', const=True, default=False, metavar="True|False")
+
     parser.add_argument(      "--presorted", dest="sorted_input",
                               help="Indicate that the input has been presorted (or at least pregrouped). (default=%(default)s).",
                               type=optional_bool, nargs='?', const=True, default=False)
@@ -402,6 +445,7 @@ def main():
         print("--output-file=%s" % str(args.output_file_path), file=error_file, flush=True)
         print("--columns %s" % " ".join(args.key_column_names), file=error_file, flush=True)
         print("--compact-id=%s" % str(args.compact_id), file=error_file, flush=True)
+        print("--deduplicate=%s" % str(args.deduplicate), file=error_file, flush=True)
         print("--presorted=%s" % str(args.sorted_input), file=error_file, flush=True)
         print("--verify-sort=%s" % str(args.verify_sort), file=error_file, flush=True)
         print("--lists-in-input=%s" % str(args.lists_in_input), file=error_file, flush=True)
@@ -414,6 +458,7 @@ def main():
         input_file_path=args.input_file_path,
         key_column_names=args.key_column_names,
         compact_id=args.compact_id,
+        deduplicate=args.deduplicate,
         sorted_input=args.sorted_input,
         verify_sort=args.verify_sort,
         lists_in_input=args.lists_in_input,
